      subroutine ffr(s,u,up)
c     ODE system for (ocean sound) REFRACTED geodesics,
c     processed by the /rkf45 system

c     subr/ffr  status :     23 March 1993

c     J. G. Dworski

c     H a n d l i n g :
c     =================
c     rmer [km] is the meridional radius of curvature of the
c     earth's oblate spheroid at latitude /glat; rpve [km] is
c     the radius of curvature in the plane of the "prime vertical",
c     at the same latitude.

c     sinl,cosl are sine and cosine of latitude. /geordG/ returns
c     the nontrivial true ellipsoidal WGS84 parametrization, and
c     is accompanied by a nearly trivial version /geordS/ that
c     returns the corresponding information on a sphere of equal
c     area to the earth's ellipsoid.

c     s      =  path-length  [km]
c     u(1)   =  latitude     [rad]
c     u(2)   =  longitude    [rad]
c     u(3)   =  azimuth      [rad]
c     up(1)  =  du(1)/ds     [rad/km]
c     up(2)  =  du(2)/ds     [rad/km]
c     up(3)  =  du(3)/ds     [rad/km]

c     -------------------------------------
      implicit double precision (a-h,o-z)
      parameter( tor=1.745329252d-02 )
      parameter( n=3,pi=3.1415926535d0,mode=1 )
      double precision       s,u(n),up(n)

      glat    =  u(1)
      glon    =  u(2)
      ylat    =  glat/tor
      xlon    =  glon/tor
c     ****************************************
      call geordG( glat,sinl,cosl,rmer,rpve )
      call lapatch(xlon,ylat,cc,dclo,dcla,mode)
c     ****************************************
      cosa    =  cos(u(3))
      sina    =  sin(u(3))

      up(1)   =  cosa /     rmer
      up(2)   =  sina /   (rpve*cosl)
c     non-perturbed azimuth derivative :
      up3     =  sina*(sinl/cosl)/rpve
c     perturbation derivative term :
      up3per  =  sina*(dcla/cc)/rmer  - cosa*(dclo/cc)/(rpve*cosl)
c     azimuth derivative :
      up(3)   =  up3 + up3per

      return
      end
