/*
   Calculate great circle distance
    Copyright (C) 1993-94  University of Washington

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Written by:  Shaun Leach

    Applied Physics Lab
    University of Washington
    1013 NE 40th St
    Seattle, WA 98105
    e-mail:  shaun@anchor.apl.washinton.edu
    phone:  (206) 543-1300
*/

#include <math.h>
#include <stdio.h>

#define deg2rad 0.0174532925
#define PI 3.1415926536

void cdist(double *lat1, double *lon1, double *lat2, double *lon2, 
           double *azis, double *azir, double *range) {

  double myLat1, myLon1, myLat2, myLon2;
  double angle;
  double theta;

  if (*lon1 < 0)
    *lon1 += 360;
  if (*lon2 < 0)
    *lon2 += 360;

  myLat1 = deg2rad * *lat1;
  myLon1 = deg2rad * *lon1;
  myLat2 = deg2rad * *lat2;
  myLon2 = deg2rad * *lon2;

  *range = sin(myLat1) * sin(myLat2) + 
    cos(myLat1) * cos(myLat2) * cos(myLon2 - myLon1);

  *range = 111120.0 * acos(*range) / deg2rad;

  theta = acos(sin(myLat1) * sin(myLat2) + 
               cos(myLat1) * cos(myLat2) * cos(myLon2 - myLon1));
  angle = acos((sin(myLat2) - sin(myLat1) * cos(theta)) / 
               (sin(theta) * cos(myLat1)));
  if (sin(myLon2 - myLon1) < 0)
    angle = 2 * PI - angle;
  *azis = angle / deg2rad;


  theta = acos(sin(myLat2) * sin(myLat1) + 
               cos(myLat2) * cos(myLat1) * cos(myLon1 - myLon2));
  angle = acos((sin(myLat1) - sin(myLat2) * cos(theta)) / 
               (sin(theta) * cos(myLat2)));
  if (sin(myLon1 - myLon2) < 0)
    angle = 2 * PI - angle;
  *azir = angle / deg2rad;
}
