/*
   Calculate points along a great circle
    Copyright (C) 1993-94  University of Washington

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Written by:  Shaun Leach

    Applied Physics Lab
    University of Washington
    1013 NE 40th St
    Seattle, WA 98105
    e-mail:  shaun@anchor.apl.washinton.edu
    phone:  (206) 543-1300
*/

#include <math.h>
#include <stdio.h>

#define deg2rad 0.0174532925
#define PI 3.1415926536

void scshoot(double *lat1, double *lon1, double *lat2, double *lon2, 
             double *azi, double *inc, double *range,
             double *ranges, double *lats, double *lons, double *azis) {

  double myLat1, myLon1, myLat2, myLon2;
  double angle;
  double theta;
  int steps, i;
  double lonInc, diffLon;

  if (*lon1 < 0)
    *lon1 += 360;
  if (*lon2 < 0)
    *lon2 += 360;

  myLat1 = deg2rad * *lat1;
  myLon1 = deg2rad * *lon1;
  myLat2 = deg2rad * *lat2;
  myLon2 = deg2rad * *lon2;

  steps = (int) (*range / *inc) + 1;

  lonInc = (myLon2 - myLon1) / (double) steps;
 
  for(i = 0; i < steps; i++) {
    lons[i] = (myLon1 + i * lonInc) / deg2rad;
    if (lonInc != 0) {
      diffLon = myLon1 + i * lonInc - myLon2;
      lats[i] = tan(myLat2) * sin(i * lonInc) - tan(myLat1) * sin(diffLon);
      lats[i] = atan(lats[i] / sin(myLon2 - myLon1)) / deg2rad;
    }
    else {
      lats[i] = myLat1 + i * ((myLat2 - myLat1) / steps);
      lats[i] = lats[i] / deg2rad;
    }
    ranges[i] = *inc * i;

    theta = acos(sin(lats[i]) * sin(myLat2) + 
                 cos(lats[i]) * cos(myLat2) * cos(myLon2 - lons[i]));
    angle = acos((sin(myLat2) - sin(lats[i]) * cos(theta)) / 
                 (sin(theta) * cos(lats[i])));
    if (sin(myLon2 - lons[i]) < 0)
      angle = 2 * PI - angle;
    azis[i] = angle / deg2rad;
  }

  lons[i] = *lon2;
  lats[i] = *lat2;
  ranges[i] = *range;
  azis[i] = azis[i - 1];
}
